@@ -6,7 +6,7 @@ use Test::Builder;
 use Test::LongString;
 use Exporter ();
 
-our $VERSION = '0.57';
+our $VERSION = '0.58';
 our @ISA     = 'Exporter';
 our @EXPORT  = qw{ code compiles slurp };
 
@@ -34,7 +34,8 @@ sub compiles {
 		if ( $ENV{ADAMK_RELEASE} ) {
 			$Test->ok( 1, "Skipped $_[0]" );
 		} else {
-			my $rv = eval $code;
+			local $@;
+			my $rv = do { eval "return 1;\n$code"; };
 			$Test->diag( $@ ) if $@;
 			$Test->ok( $rv, $_[0] );
 		}
